//////////////////////////////////////////////////////////////
//	FILE: graphicsMath.h
//  created:  Oct 12 2009 by M. van de Panne, UBC
//////////////////////////////////////////////////////////////
 
#ifdef WIN32
#define M_PI 3.1415926
#endif

class VEC3 {             // 3D vector
public:
	float v[3];
	VEC3() {  v[0]=v[1]=v[2]=0.0;};                                    // constructor: init to zero
	VEC3(float x, float y, float z) { v[0]=x; v[1]=y; v[2]=z; };       // constructor: init to x,y,z
	void print(const char *name) { printf("%s:  %6.3f %6.3f %6.3f \n",name,v[0],v[1],v[2]); };   // print
	float len() { return (float) sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]); };   // length, i.e., two norm
	float dot(VEC3 &a) { return v[0]*a.v[0] + v[1]*a.v[1] + v[2]*a.v[2]; };    // dot product
	void zero() {  v[0]=v[1]=v[2]=0.0; };                              // set to zero
	bool normalize() {  
		float size=len();
		if (size==0.0) return false;
		float scale = 1.0/size;
		v[0]*=scale; v[1]*=scale; v[2]*=scale;
		return true;
	}
	VEC3 operator+(const VEC3 a) const {                         // vector addition
		return VEC3( v[0]+a.v[0], v[1]+a.v[1], v[2]+a.v[2]); };    
	VEC3 operator-(const VEC3 a) const {                         // vector subtraction
		return VEC3( v[0]-a.v[0], v[1]-a.v[1], v[2]-a.v[2]); };    
	VEC3 operator+=(const VEC3 a) {                              // vector self-addition
		return VEC3( v[0]+=a.v[0], v[1]+=a.v[1], v[2]+=a.v[2]); };    
	VEC3 operator-=(const VEC3 a) {                              // vector self-subtraction
		return VEC3( v[0]-=a.v[0], v[1]-=a.v[1], v[2]-=a.v[2]); };    
	VEC3 operator*(const VEC3 a) const {                         // cross product
		return VEC3( 
			v[1]+a.v[2] - v[2]*a.v[1],
			v[2]*a.v[0] - v[0]*a.v[2],
			v[0]*a.v[1] - v[1]*a.v[0]
		); 
	};
};


class TMATRIX {           //  4x4 transformation matrix
public:
	float m[4][4];
	TMATRIX();                                // constructor:  initialize to identity
	TMATRIX(VEC3 i, VEC3 j, VEC3 k, VEC3 p);  // constructor:  initialize to given frame
	void iden();                          // set to identity
	VEC3 operator*(const VEC3 a);         // multiply by a vector
	TMATRIX operator*(const TMATRIX a);   // multiply by a matrix
	void print(const char *name);               // print matrix, with title line
	void rotkth(VEC3 k, float th);        // set to Rot(k,th)
	void translate(VEC3 t);               // set to Translate matrix
	VEC3 getcol(int c);                   // get column from matrix
	void quat(float q[4]);                // set to Rot(q) where q is a unit quaternion
	void getkth(VEC3 &k, float &th);      // get (k,theta) from a rotation matrix

};
