///////////////////////////////////////////////////////////////////
// FILE:      main.cpp
// CONTAINS:  template code for CPSC 426, Jan 2014, Assignment 1
///////////////////////////////////////////////////////////////////

#include "defs.h"

float Fovy = 35.0;    // field of view  in the y-direction (in degrees)
int Width = 400;      // window width (pixels)
int Height = 400;     // window height (pixels)
float Znear = 1.0;    // near clipping plane (distance from camera)
float Zfar = 100.0;   // far clipping plane (distance from camera)
int iCount = 0;       // used for numbering the PPM image files
float Time = 0;       // initial time
float MaxTime = 1.5;  // maximum time
float Dt = 0.01;      // time step
float AnimTimeStep = 0.01;

void computeLocation();     // computes location(s) of objects
void drawCube(float sx, float sy, float sz);    // draw cube of given size

GLfloat SphereLocation[4];

///////////////////////////////////////////////////////////////////
// FUNC:  displayCallback()
// DOES:  clears screen, draws the world, swaps buffers
///////////////////////////////////////////////////////////////////

void displayCallback(void)
{
	float eye[] = {-6,8,20};
	float center[] = {0,0,0};
	float up[] = {0,1,0};

		// clear drawing buffer
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	  // setup ModelView matrix with the viewmatrix
	  // i.e., set the camera position
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(eye[0],eye[1],eye[2],
		center[0],center[1],center[2],
		up[0],up[1], up[2]);

	  // draw sphere
	glPushMatrix();
	glColor3f(0.5,0.7,0.6);
	glTranslatef(SphereLocation[0],SphereLocation[1],SphereLocation[2]);
	glutSolidSphere(1,20,10);     // sphere: r=1, 20 slices, 10 stacks
	glPopMatrix();

	glPushMatrix(); 
	  // draw some colored teapots
	glTranslatef(-3,0,1);
	glColor3f(1,0.5,0.5);      // mostly red
	glutSolidTeapot(1.0);

	glTranslatef(3,0,0);
	glColor3f(0.5,1,0.5);      // mostly green
	glutSolidTeapot(1.0);
	
	glTranslatef(3,0,0);
	glColor3f(0.5,0.5,1);      // mostly blue
	glutSolidTeapot(1.0);

	glPopMatrix();

	  // draw ground plane
	glColor3f(1,1,1);          // white (greyish after lighting)
	glNormal3f(0,1,0);
	glBegin(GL_POLYGON);
	glVertex3f(-10,0,-10);
	glVertex3f(-10,0,10);
	glVertex3f(10,0,10);
	glVertex3f(10,0,-10);
	glEnd();

	glutSwapBuffers();
}

///////////////////////////////////////////////////////////////////
// FUNC:  keyCallback()
// DOES:  handles glut keyboard events
///////////////////////////////////////////////////////////////////

void keyCallback(unsigned char key, int x, int y)
{
	switch(key) {
	case 'q': 
		exit(0);
		break;
	case 'r':
		Time = 0;               // set global time to zero
		Dt = AnimTimeStep;      // time step
		break;
	case ' ':
                if (Dt==AnimTimeStep)
		  Dt = 0;             // stop animation
		else 
		  Dt = AnimTimeStep;  // start animation
		break;
	default:
		break;
	}
	glutPostRedisplay();
}

///////////////////////////////////////////////////////////////////
// FUNC:  computeLocation()
// DOES:  computes location of objects
///////////////////////////////////////////////////////////////////

void computeLocation()
{
  float angle = M_PI*Time;
  SphereLocation[0] = 3.0*sin(angle);
  SphereLocation[1] = 1.5;
  SphereLocation[2] = 3.0*cos(angle);
}

///////////////////////////////////////////////////////////////////
// FUNC:  myReshape()
// DOES:  this is called if the on-screen window is resized
///////////////////////////////////////////////////////////////////

void myReshape(int w, int h)
{
   Width = w;
   Height = h;
   glViewport(0, 0, w, h);
   glMatrixMode(GL_PROJECTION); 
   glLoadIdentity();
   gluPerspective(Fovy, (GLfloat) w/h, 1.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}

///////////////////////////////////////////////////////////////////
// FUNC:  idleCallback()
// DOES:  glut calls this when idle;  used for animation
///////////////////////////////////////////////////////////////////

void idleCallback()
{
	if (Dt==0) return;        // skip if not animating
	Time += Dt;    // increment current time by Dt (default Dt=0)
	if (Time>=MaxTime)
	  Time = 0.0;
//	printf("  Time=%6.3f\n",Time);

	  // interpolate animation parameters for current parameters
	computeLocation();

	glutPostRedisplay();
}

///////////////////////////////////////////////////////////////////
// FUNC:  init()
// DOES:  initializes default light source and surface material
///////////////////////////////////////////////////////////////////

void init(void) 
{
   glClearColor (0.2f, 0.3f, 0.4f, 1.0);
   glShadeModel (GL_SMOOTH);

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_NORMALIZE);

   SphereLocation[0] = SphereLocation[1] = SphereLocation[2] = 0.5;

}

///////////////////////////////////////////////////////////////////
// FUNC:  main()
// DOES:  intializes glut window, calls init(), then hands over control to glut
///////////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(Width, Height);
   glutCreateWindow("CPSC 426 Jan 2014:  Assmt 1");
   glutReshapeFunc(myReshape);
   glutDisplayFunc(displayCallback);
   glutKeyboardFunc(keyCallback);
   glutIdleFunc(idleCallback);
   init();
   glutMainLoop();
   return 0;
}


///////////////////////////////////////////////////////////////////
// FUNC:  drawCube()
// DOES:  draws a cube of unit size, with x,y in [-0.5,0.5] and z in [0,1]
//
//                 5------------------6
//                /.                 /|
//               / .                / |
//              /  .               /  |
//             /   .              /   |
//            /    .             /    |
//           /     .            /     |
//          /      .           /      |
//         3------------------2       |
//         |       .          |       |
//         |       4..........|.......7
//         |      .           |      / 
//         |     .    y|      |     /  
//         |    .      |      |    /   
//         |   .       *----x |   /    
//         |  .       /       |  /     
//         | .     z /        | /      
//         |.                 |/       
//         0------------------1
//     
///////////////////////////////////////////////////////////////////																										

void drawCube(float sx, float sy, float sz)
{
	const float MINX = -0.5;
	const float MAXX = 0.5;
	const float MINY = 0;
	const float MAXY = 1;
	const float MINZ = -0.5;
	const float MAXZ = 0.5;

	  // 8 cube vertices
	float vertex[8][3] = {
		MINX, MINY, MAXZ,   // 0
		MAXX, MINY, MAXZ,   // 1
		MAXX, MAXY, MAXZ,   // 2 
		MINX, MAXY, MAXZ,   // 3
		MINX, MINY, MINZ,   // 4
		MINX, MAXY, MINZ,   // 5
		MAXX, MAXY, MINZ,   // 6
		MAXX, MINY, MINZ,   // 7
	};
	int faceV[6][4] = {      // vertices in counter-clockwise order
		0,1,2,3,   // front face
	    4,5,6,7,   // back face
		1,7,6,2,   // right face
		0,3,5,4,   // left face
		3,2,6,5,   // top face
		1,0,4,7    // bottom face
	};
	float faceN[6][3] = {
		
	    0,0,1,    // front face
		0,0,-1,   // back face
		1,0,0,    // right face
		-1,0,0,   // left face
		0,1,0,    // top face
		0,-1,0,   // bottom face
	};

	glPushMatrix();
	glScalef(sx,sy,sz);     // scale cube to the desired size
	for (int f=0; f<6; f++) {
		glBegin(GL_POLYGON);
		  // 3D normal (used by OpenGL for lighting)
		  // This becomes the 'current normal' that is assigned
		  // to all the following vertices
		glNormal3fv( faceN[f] );		                       
		for (int v=0; v<4; v++) {
			int i = faceV[f][v];      // vertex number
			glVertex3fv( vertex[i] );  // 3D vertex coords
		}
		glEnd();
	}
	glPopMatrix();
}

